use clokwerk::{Scheduler, TimeUnits};
use feed::should_feed_be_updated;
use remaining_place::{get_current_places, RemainingPlace};
use std::sync::{Arc, Mutex};
use std::thread;
use std::time::Duration;

use crate::feed::update_rss_file;

mod feed;
mod html_parser;
mod remaining_place;

struct Args {
    url: String,
    rss_file: String,
}

fn parse_args() -> Result<Args, pico_args::Error> {
    let mut pargs = pico_args::Arguments::from_env();

    let url = pargs.opt_value_from_str("--url")?;

    let args = Args {
        url: match url {
            Some(val) => val,
            None => String::from("https://www.lfs.sachsen.de/restplatzboerse-5152.html"),
        },
        rss_file: pargs.value_from_str("--rss-file")?,
    };

    Ok(args)
}

fn main() {
    let args = match parse_args() {
        Ok(v) => v,
        Err(err) => {
            eprintln!("Error: {err}");
            std::process::exit(1);
        }
    };

    let last_places: Arc<Mutex<Vec<RemainingPlace>>> = Arc::new(Mutex::new(Vec::new()));
    let mut scheduler = Scheduler::new();

    let check_for_places = move || {
        match get_current_places(&args.url) {
            Ok(places) => {
                if should_feed_be_updated(&places, &last_places) {
                    update_rss_file(places, &args.url, &args.rss_file);
                    println!("Updated feed.");
                } else {
                    println!("No update.");
                }
            }
            Err(error) => {
                println!("Error: {}", error);
            }
        };
    };

    // now
    check_for_places();

    // and every 30 min
    scheduler.every(30.minutes()).run(check_for_places);

    loop {
        scheduler.run_pending();
        thread::sleep(Duration::from_millis(10));
    }
}
